// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_POSIX_WAITABLE_H
#define HU_VARADIISTVAN_SCPL_IO_POSIX_WAITABLE_H
//------------------------------------------------------------------------------

#include "../Failable.h"

#include <cerrno>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

class Waiter;

//------------------------------------------------------------------------------

/**
 * Base class of objects for which one can wait in a Waiter.
 *
 * It is basically a file descriptor.
 */
class Waitable : public Failable
{
public:
    /**
     * Set the given file descriptor to non-blocking.
     */
    static void setNonBlocking(int fd);

protected:
    /**
     * The waiter this waitable belongs to, if any.
     */
    Waiter* waiter;

    /**
     * The file descriptor
     */
    int fd;

    /**
     * The events this waitable is interested in.
     */
    short events;

protected:
    /**
     * Construct the waitable.
     */
    Waitable(Waiter* waiter, int fd = -1, short events = 0);

public:
    /**
     * Destroy the waitable. It will be removed from the waiter.
     */
    virtual ~Waitable();

    /**
     * Get the waiter.
     */
    Waiter* getWaiter() const;

    /**
     * Determine if the waitable is ready or not, i.e. if there is no
     * need to wait for it.
     *
     * It first checks the events. If they are 0, the waitable is
     * ready. Then it calls isReady() and returns the result of that
     * function.
     */
    bool ready();

protected:
    /**
     * Indicate if the waitable is ready, i.e. there is no need to
     * wait for it.
     *
     * Calling this function should not delete the ready
     * indication. It should be deleted by actually processing the
     * data.
     */
    virtual bool isReady() = 0;

    /**
     * Handle the given events.
     */
    virtual void handleEvents(short events) = 0;

    /**
     * Set the error code from errno.
     */
    void setErrorCodeFromErrno();

    friend class Waiter;
};

//------------------------------------------------------------------------------
// Inline defintions
//------------------------------------------------------------------------------

inline Waiter* Waitable::getWaiter() const
{
    return waiter;
}

//------------------------------------------------------------------------------

inline bool Waitable::ready()
{
    return (events==0) || isReady();
}

//------------------------------------------------------------------------------

inline void Waitable::setErrorCodeFromErrno()
{
    setErrorCode(errno);
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_POSIX_WAITABLE_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
